import os
import re
from datetime import datetime

FILE_EXTENSIONS = ['.py', '.c', '.js', '.java', '.pl', '.go', '.lua', '.mod', '.hs', '.yaws', '.sh',
'.d', '.rs']
INCLUDED_DIRS = ['programming', 'code', 'projects']
EXCLUDED_DIRS = ['node_modules', 'github.com', '.git', '.github']

data = {} #structured with keys being the username, values being an array

def lines(file_name):
    count = 0
    with open(file_name, 'r') as f:
        for line in f:
            count += 1
    return count

#return what elements of two lists overlap (case-insensitie)
def overlap(list1, list2):
    if set(list1).isdisjoint(list2):
        return []
    else:
        return [value for value in list2 if value.lower() in list1]

#return the LOC in a search set, and the files in that set (filter by extension)
def search_files(search_set, only_files=True):
    total = 0
    file_set = []
    for root, dirs, files in search_set:
        if only_files: [dirs.remove(d) for d in dirs]
        else: [dirs.remove(d) for d in list(dirs) if d in EXCLUDED_DIRS]
        for name in files:
            f = ''
            try:
                f = os.path.join(root, name)
                if f.endswith(tuple(FILE_EXTENSIONS)):
                    file_set.append(f)
                    total += lines(f)
            except:
                continue 
    return total, file_set

#searches through string,and overwrites substring between two tags
def overwrite_tag(string, tag, overwrite):
    result = re.search('(?<=<!--%s-->)([\s\S]*)(?=<!--\/%s-->)' % (tag, tag), string)
    return string.replace(result.group(1), overwrite)

#get the directories in a user's project folders, and their respective LOC
#get the total LOC in their project folders
for user in os.listdir('/home'):
    user_dirs = ''
    try:
        user_dirs = list(os.listdir('/home/' + user))
    except:
        continue

    #test if the user has any of the included folders:
    for i in overlap(INCLUDED_DIRS, user_dirs):
        user_files = []
        path = '/home/' + user + '/' + i
        project_dirs = [value for value in next(os.walk(path))[1] if value not in EXCLUDED_DIRS]
        project_dirs_dict = {}

        # get the LOC for each of their project dirs
        for folder in project_dirs:
            total = search_files(os.walk(path + "/" + folder, topdown=True), False)[0]
            project_dirs_dict[folder] = total
        
        # get the LOC for the individual files, and add total for directories
        misc_total = search_files(os.walk(path, topdown=True))[0]
        total = misc_total + sum(project_dirs_dict.values())

        data[user] = [project_dirs_dict, total, misc_total]

for key in [key for key in data if data[key][1] == 0]: del data[key] #prune entires where the total is 0
#data = sorted(data.items(), key=lambda x: x[1][1], reverse=True) #sort by greatest total
data = sorted(data.items(), key=lambda x: x[0])
#^sorting by greatest total could create the idea that it's competitive, which it shouldn't be, so here's alphabetical

f = open("/home/pine/public_html/loc.html", "r")
html = f.read()
f.close()

html = overwrite_tag(html, 'time', str(datetime.now()))

loc_list = ''
for i in data:
    loc_list += '<li><a href="/~%s">~%s</a> &mdash; wrote %s lines of code</li>' % (i[0], i[0], str(i[1][1]))
    if i[1][0].items():
        loc_list += "\n<ul>"
        for dir_name, total in i[1][0].items():
            loc_list += "\n\t<li><code>%s</code> &mdash; %s lines</li>" % (dir_name, str(total))
        if i[1][2] > 0:
            loc_list += "\n\t<li>misc files &mdash; %s lines</li>" % str(i[1][2])
        loc_list += "\n</ul>\n"

html = overwrite_tag(html, 'list', loc_list)
f = open("/home/pine/public_html/loc.html", "w")
f.write(html)
f.close()
